from zope.interface import Interface


class IValidationService(Interface):

    def validate(name_or_validator, value, *args, **kwargs):
        """call the validator of a given name"""

    def validatorFor(name_or_validator):
        """return the validator for a given name"""

    def register(validator):
        """load a validator for access by name"""

    def unregister(name_or_validator):
        """unregisters a validator by name"""



from Products.validation.i18n import PloneMessageFactory as _
from Products.validation.i18n import recursiveTranslate
from Products.validation.i18n import safe_unicode
from Products.validation.interfaces.IValidator import IValidator
from zope.interface import implementer


@implementer(IValidator)
class RangeValidator:

    def __init__(self, name, minval=0.0, maxval=0.0, title="", description=""):
        self.name = name
        self.minval = minval
        self.maxval = maxval
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        if len(args) >= 1:
            minval = args[0]
        else:
            minval = self.minval

        if len(args) >= 2:
            maxval = args[1]
        else:
            maxval = self.maxval

        assert minval <= maxval
        try:
            nval = float(value)
        except ValueError:
            msg = _(
                "Validation failed($name): could not convert '$value' to number",
                mapping={"name": safe_unicode(self.name), "value": safe_unicode(value)},
            )
            return recursiveTranslate(msg, **kwargs)
        if minval <= nval < maxval:
            return 1

        msg = _(
            "Validation failed($name): '$value' out of range($min, $max)",
            mapping={
                "name": safe_unicode(self.name),
                "value": safe_unicode(value),
                "min": safe_unicode(minval),
                "max": safe_unicode(maxval),
            },
        )
        return recursiveTranslate(msg, **kwargs)



from AccessControl import ClassSecurityInfo
from AccessControl import ModuleSecurityInfo
from AccessControl.class_init import InitializeClass
from Acquisition import Implicit
from Products.validation.service import Service


# make validator service public
security = ModuleSecurityInfo("Products.validation.config")
security.declarePublic("validation")


class ZService(Service, Implicit):
    """Service running in a zope site - exposes some methods"""

    security = ClassSecurityInfo()

    security.declarePublic("validate")
    security.declarePublic("__call__")
    security.declarePublic("validatorFor")


InitializeClass(ZService)



from Acquisition import aq_base
from Acquisition import aq_get
from Acquisition import aq_inner
from Acquisition import aq_parent
from OFS import ObjectManager
from Products.validation.i18n import PloneMessageFactory as _
from Products.validation.i18n import recursiveTranslate
from Products.validation.i18n import safe_unicode
from Products.validation.interfaces.IValidator import IValidator
from zExceptions import BadRequest
from zope.interface import implementer


def fallback_check_id(instance, id, **kwargs):
    # space test
    if " " in id:
        msg = _("Spaces are not allowed in ids")
        return recursiveTranslate(msg, **kwargs)

    # in parent test
    parent = aq_parent(aq_inner(instance))
    # If the id is given to a different object already
    if id in parent.objectIds() and getattr(aq_base(parent), id) is not aq_base(
        instance
    ):
        msg = _("Id $id is already in use", mapping={"id": safe_unicode(id)})
        return recursiveTranslate(msg, **kwargs)

    # object manager test
    try:
        # Note: we used to pass 'self' (the validator) instead of 'instance',
        # which makes no sense.
        ObjectManager.checkValidId(instance, id, allow_dup=1)
    except BadRequest as m:
        return str(m)
    return 1


@implementer(IValidator)
class IdValidator:

    def __init__(self, name, title="", description=""):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, id, instance, *args, **kwargs):
        # Try to get a check_id from the instance,
        # for example a Python skin script or a method,
        # like Products/CMFPlone/skins/plone_scripts/check_id.py
        # until Plone 5.1.
        check_id = aq_get(instance, "check_id", None, 1)
        if check_id is not None:
            # instance is passed implicitly: it is 'self'
            result = check_id(id, required=kwargs.get("required"))
        else:
            try:
                # try to use the check_id script from plone.base.utils.
                # Import here to avoid a hard dependency.
                from plone.base.utils import check_id
            except ImportError:
                # Use our own fallback function.
                check_id = fallback_check_id
                # Pass all keywords.  We may need the request.
                result = check_id(instance, id, **kwargs)
            else:
                # Only the 'required' keyword is accepted.
                result = check_id(instance, id, required=kwargs.get("required"))
        return result or 1


validatorList = [
    IdValidator("isValidId", title="", description=""),
]

__all__ = ("validatorList",)



# backward compatibility
from Products.validation.interfaces.IValidationService import (
    IValidationService as ivalidationService,
)
from Products.validation.interfaces.IValidator import IValidator as ivalidator



from Products.validation.i18n import PloneMessageFactory as _
from Products.validation.i18n import recursiveTranslate
from Products.validation.i18n import safe_unicode
from Products.validation.interfaces.IValidator import IValidator
from zope.interface import implementer


_marker = []


@implementer(IValidator)
class EmptyValidator:

    def __init__(self, name, title="", description="", showError=True):
        self.name = name
        self.title = title or name
        self.description = description
        self.showError = showError

    def __call__(self, value, *args, **kwargs):
        isEmpty = kwargs.get("isEmpty", False)
        instance = kwargs.get("instance", None)
        field = kwargs.get("field", None)

        # XXX: This is a temporary fix. Need to be fixed right for AT 2.0
        #      content_edit / BaseObject.processForm() calls
        #      widget.process_form a second time!
        if instance and field:
            widget = field.widget
            request = getattr(instance, "REQUEST", None)
            if request and request.form:
                form = request.form
                result = widget.process_form(
                    instance, field, form, empty_marker=_marker, emptyReturnsMarker=True
                )
                if result is _marker or result is None:
                    isEmpty = True

        if isEmpty:
            return True
        elif value == "" or value is None:
            return True
        else:
            if getattr(self, "showError", False):
                msg = _(
                    "Validation failed($name): '$value' is not empty.",
                    mapping={
                        "name": safe_unicode(self.name),
                        "value": safe_unicode(value),
                    },
                )
                return recursiveTranslate(msg, **kwargs)
            else:
                return False


validatorList = [
    EmptyValidator("isEmpty", title="", description=""),
    EmptyValidator("isEmptyNoError", title="", description="", showError=False),
]

__all__ = ("validatorList",)



from Acquisition import aq_base
from DateTime import DateTime
from io import TextIOWrapper
from Products.validation.i18n import PloneMessageFactory as _
from Products.validation.i18n import recursiveTranslate
from Products.validation.i18n import safe_unicode
from Products.validation.interfaces.IValidator import IValidator
from zope.interface import implementer
from ZPublisher.HTTPRequest import FileUpload


_marker = []


@implementer(IValidator)
class MaxSizeValidator:
    """Tests if an upload, file or something supporting len() is smaller than a
       given max size value

    If it's a upload or file like thing it is using seek(0, 2) which means go
    to the end of the file and tell() to get the size in bytes otherwise it is
    trying to use len()

    The maxsize can be acquired from the kwargs in a call, a
    getMaxSizeFor(fieldname) on the instance, a maxsize attribute on the field
    or a given maxsize at validator initialization.
    """

    def __init__(self, name, title="", description="", maxsize=0):
        self.name = name
        self.title = title or name
        self.description = description
        self.maxsize = maxsize

    def __call__(self, value, *args, **kwargs):
        instance = kwargs.get("instance", None)
        field = kwargs.get("field", None)

        # get max size
        if kwargs.has_key("maxsize"):
            maxsize = kwargs.get("maxsize")
        elif hasattr(aq_base(instance), "getMaxSizeFor"):
            maxsize = instance.getMaxSizeFor(field.getName())
        elif hasattr(field, "maxsize"):
            maxsize = field.maxsize
        else:
            # set to given default value (default defaults to 0)
            maxsize = self.maxsize

        if not maxsize:
            return True

        # calculate size
        elif (
            isinstance(value, FileUpload)
            or isinstance(value, TextIOWrapper)
            or hasattr(aq_base(value), "tell")
        ):
            value.seek(0, 2)  # eof
            size = value.tell()
            value.seek(0)
        else:
            try:
                size = len(value)
            except TypeError:
                size = 0
        size = float(size)
        sizeMB = size / (1024 * 1024)

        if sizeMB > maxsize:
            msg = _(
                "Validation failed($name: Uploaded data is too large: ${size}MB (max ${max}MB)",
                mapping={
                    "name": safe_unicode(self.name),
                    "size": safe_unicode("%.3f" % sizeMB),
                    "max": safe_unicode("%.3f" % maxsize),
                },
            )
            return recursiveTranslate(msg, **kwargs)
        else:
            return True


@implementer(IValidator)
class DateValidator:

    def __init__(self, name, title="", description=""):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        if not value:
            msg = _(
                "Validation failed($name): value is empty ($value).",
                mapping={"name": self.name, "value": repr(value)},
            )
            return recursiveTranslate(msg, **kwargs)
        if not isinstance(value, DateTime):
            try:
                value = DateTime(value)
            except Exception:
                msg = _(
                    "Validation failed($name): could not convert $value to a date.",
                    mapping={
                        "name": safe_unicode(self.name),
                        "value": safe_unicode(value),
                    },
                )
                return recursiveTranslate(msg, **kwargs)
        return True


validatorList = [
    MaxSizeValidator("isMaxSize", title="", description=""),
    DateValidator("isValidDate", title="", description=""),
]

__all__ = ("validatorList",)



class ValidatorError(Exception):
    pass


class UnknowValidatorError(ValidatorError):
    pass


class FalseValidatorError(ValidatorError):
    pass


class AlreadyRegisteredValidatorError(ValidatorError):
    pass



from zope.i18n import translate
from zope.i18nmessageid import MessageFactory


PloneMessageFactory = MessageFactory("plone")


def safe_unicode(value):
    return value


def recursiveTranslate(message, **kwargs):
    """translates also the message mappings before translating the message.

    if kwargs['REQUEST'] is None, return the message untranslated

    Actually, recursive translation has been built into zope.i18n 3.5.0,
    which was already released in 2008.  See
    https://github.com/zopefoundation/zope.i18n/blob/master/CHANGES.rst#350-2008-07-10

    So we can simply call the translate function.
    This avoids a TypeError in Zope 5.11+, as `map[key]` is immutable there.
    """
    request = kwargs.get("REQUEST", None)
    return translate(message, context=request)



##############################################################################
#
# TALESField - Field with TALES support for Archetypes
# Copyright (C) 2005 Sidnei da Silva, Daniel Nouri and contributors
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

from Products.PageTemplates.Expressions import getEngine
from Products.validation.i18n import PloneMessageFactory as _
from Products.validation.i18n import recursiveTranslate
from Products.validation.i18n import safe_unicode
from Products.validation.interfaces.IValidator import IValidator
from zope.i18nmessageid import Message
from zope.interface import implementer


@implementer(IValidator)
class ExpressionValidator:
    """Validator for TALES Expressions

    Basically, if the expression compiles it's a valid expression,
    otherwise it's invalid and you get a message saying that the
    expression has errors.


    >>> val=ExpressionValidator("python: int(value) == 5")
    >>> class C:i=1
    >>> c=C()
    >>> val(5,c) is True
    True

    now lets fail a test
    >>> str(val(4,c))
    'validation failed, expr was:python: int(value) == 5'

    It is also possible to specify the error string

    >>> val=ExpressionValidator("python: int(value) == 5", "value does not match %(value)s")
    >>> val(4,c)
    'value does not match 4'


    """

    name = "talesexpressionvalidator"

    def __init__(self, expression=None, errormsg=None):
        self.expression = expression
        self.errormsg = errormsg
        if expression:
            self.compiledExpression = getEngine().compile(expression)

    def __call__(self, value, instance, *args, **kwargs):
        kw = {
            "here": instance,
            "object": instance,
            "instance": instance,
            "value": value,
            "args": args,
            "kwargs": kwargs,
        }

        context = getEngine().getContext(kw)
        res = self.compiledExpression(context)

        if res:
            return True
        else:
            if self.errormsg and isinstance(self.errormsg, Message):
                # hack to support including values in i18n message, too. hopefully this works out
                # potentially it could unintentionally overwrite already present values
                self.errormsg.mapping = kw
                return recursiveTranslate(self.errormsg, **kwargs)
            elif self.errormsg:
                # support strings as errormsg for backward compatibility
                return self.errormsg % kw
            else:
                msg = _(
                    "validation failed, expr was:$expr",
                    mapping={"expr": safe_unicode(self.expression)},
                )
                return recursiveTranslate(msg, **kwargs)


# validation.register(TALESExpressionValidator())



from Products.validation import validation

import unittest


class Dummy:
    """Dummy object with basic zope-like containment."""

    portal_type = "dummy"  # needed in Plone 5.2 test

    def __init__(self, _id=None):
        self.id = _id
        self._ids = []
        self._items = []

    def add(self, item):
        self._ids.append(item.id)
        self._items.append(item)
        item.__parent__ = self

    def getId(self):
        return self.id

    def getParentNode(self):
        return self.__parent__

    def objectIds(self):
        return self._ids

    def __contains__(self, id):
        return id in self._ids

    def __getattr__(self, name, default=None):
        if name in self._ids:
            for obj in self._items:
                if obj.getId() == name:
                    return obj
            return default
        return super().__getattr__(name, default)

    def dummy_checker(self, _id, **kwargs):
        if _id == "good":
            return 1
        return "bad id"


class TestValidation(unittest.TestCase):

    def test_inNumericRange(self):
        v = validation.validatorFor("inNumericRange")
        self.assertEqual(v(10, 1, 20), 1)
        self.assertEqual(v("10", 1, 20), 1)
        self.assertEqual(
            v(0, 4, 5), "Validation failed(inNumericRange): '0' out of range(4, 5)"
        )

    def test_isDecimal(self):
        v = validation.validatorFor("isDecimal")
        self.assertEqual(v("1.5"), 1)
        self.assertEqual(v("1,5"), 1)
        self.assertEqual(
            v("NaN"), "Validation failed(isDecimal): 'NaN' is not a decimal number."
        )

    def test_isPrintable(self):
        v = validation.validatorFor("isPrintable")
        self.assertEqual(v("text"), 1)
        self.assertEqual(
            v("\\u203"),
            "Validation failed(isPrintable): '\\u203' contains unprintable characters",
        )
        int_type = "<class 'int'>"
        self.assertEqual(
            v(10),
            f"Validation failed(isPrintable): 10 of type {int_type}, expected 'string'",
        )

    def test_isSSN(self):
        v = validation.validatorFor("isSSN")
        self.assertEqual(v("111223333"), 1)
        self.assertEqual(v("111-22-3333", ignore=r"-"), 1)

    def test_isUSPhoneNumber(self):
        v = validation.validatorFor("isUSPhoneNumber")
        self.assertEqual(v("(212) 555-1212", ignore=r"[\s\(\)\-]"), 1)
        self.assertEqual(v("2125551212", ignore=r"[\s\(\)\-]"), 1)

        self.assertEqual(v("(212) 555-1212"), 1)

    def test_isURL(self):
        v = validation.validatorFor("isURL")
        self.assertEqual(v("http://foo.bar:8080/manage"), 1)
        self.assertEqual(v("https://foo.bar:8080/manage"), 1)
        self.assertEqual(v("https://be.brussels:8080/manage"), 1)
        self.assertEqual(v("irc://tiran@irc.freenode.net:6667/#plone"), 1)
        self.assertEqual(v("fish://tiran:password@myserver/~/"), 1)
        self.assertEqual(
            v("http://\n"), "Validation failed(isURL): 'http://\n' is not a valid url."
        )
        self.assertNotEqual(v("../foo/bar"), 1)

    def test_isEmail(self):
        v = validation.validatorFor("isEmail")
        self.assertEqual(v("test@test.com"), 1)
        self.assertEqual(v("test@be.brussels"), 1)
        self.assertNotEqual(v("@foo.bar"), 1)
        self.assertEqual(
            v("me"), "Validation failed(isEmail): 'me' is not a valid email address."
        )

    def test_isMailto(self):
        v = validation.validatorFor("isMailto")
        self.assertEqual(v("mailto:test@test.com"), 1)
        self.assertEqual(v("mailto:test@be.brussels"), 1)
        self.assertNotEqual(v("test@test.com"), 1)
        self.assertNotEqual(v("mailto:@foo.bar"), 1)
        self.assertNotEqual(v("@foo.bar"), 1)
        self.assertNotEqual(v("mailto:"), 1)
        self.assertEqual(
            v("me"), "Validation failed(isMailto): 'me' is not a valid email address."
        )

    def test_isUnixLikeName(self):
        v = validation.validatorFor("isUnixLikeName")
        self.assertEqual(v("abcd"), 1)
        self.assertTrue(v("a_123456"), 1)
        self.assertNotEqual(v("123"), 1)
        self.assertNotEqual(v("ab.c"), 1)
        self.assertEqual(
            v("ab,c"),
            "Validation failed(isUnixLikeName): 'ab,c' this name is not a valid identifier",
        )
        self.assertNotEqual(v("aaaaaaaab"), 1)  # too long

    def test_isValidId_basic(self):
        v = validation.validatorFor("isValidId")
        obj = Dummy("foo")

        # Use a specific checker.
        obj.check_id = obj.dummy_checker
        self.assertEqual(v("good", obj), 1)
        self.assertEqual(v("a b", obj), "bad id")

    def test_isValidId_plone(self):
        try:
            # We do not use this import directly, but if it fails,
            # this test is useless.
            from plone.base.utils import check_id  # noqa F401
        except ImportError:
            return

        v = validation.validatorFor("isValidId")
        obj = Dummy("foo")
        obj2 = Dummy("foo2")
        parent = Dummy("parent")
        parent.add(obj)
        parent.add(obj2)

        self.assertEqual(v("good", obj), 1)
        self.assertEqual(v("foo", obj), 1)
        # This error message would be translated usually, but we do not care.
        self.assertEqual(
            v("foo", obj2), "There is already an item named ${name} in this folder."
        )
        # Plone seems to allow spaces.
        self.assertEqual(v("a b", obj), 1)
        # Some ids are forbidden in Plone.  We get an i18n message back.
        self.assertEqual(v("layout", obj), "${name} is reserved.")

    def test_isValidId_fallback(self):
        # We can only check this if utils.check_id gives an ImportError.
        try:
            from plone.base.utils import check_id as plone_check_id
        except ImportError:
            plone_check_id = None
        else:
            import plone.base.utils

            del plone.base.utils.check_id

        try:
            v = validation.validatorFor("isValidId")
            obj = Dummy("foo")
            parent = Dummy("parent")
            parent.add(obj)
            self.assertEqual(v("good", obj), 1)
            self.assertEqual(v("foo", obj), 1)
            self.assertEqual(v("a b", obj), "Spaces are not allowed in ids")
        finally:
            if plone_check_id:
                plone.base.utils.check_id = plone_check_id



from Products.validation.exceptions import FalseValidatorError
from Products.validation.exceptions import UnknowValidatorError
from Products.validation.interfaces.IValidationService import IValidationService
from Products.validation.interfaces.IValidator import IValidator
from zope.interface import implementer


@implementer(IValidationService)
class Service:

    def __init__(self):
        self._validator = {}

    def validate(self, name_or_validator, value, *args, **kwargs):
        v = self.validatorFor(name_or_validator)
        return v(value, *args, **kwargs)

    __call__ = validate

    def validatorFor(self, name_or_validator):
        if isinstance(name_or_validator, str):
            try:
                return self._validator[name_or_validator]
            except KeyError:
                raise UnknowValidatorError(name_or_validator)
        elif IValidator.providedBy(name_or_validator):
            return name_or_validator
        else:
            raise FalseValidatorError(name_or_validator)

    def register(self, validator):
        if not IValidator.providedBy(validator):
            raise FalseValidatorError(validator)
        name = validator.name
        # The following code prevents refreshing
        # if self._validator.has_key(name):
        #    raise AlreadyRegisteredValidatorError, name
        self._validator[name] = validator

    def items(self):
        return self._validator.items()

    def keys(self):
        return [k for k, v in self.items()]

    def values(self):
        return [v for k, v in self.items()]

    def unregister(self, name_or_validator):
        if isinstance(name_or_validator, str):
            name = name_or_validator
        elif IValidator.implementedBy(name_or_validator):
            name = name_or_validator.name
        if self._validator.has_key(name):
            del self._validator[name]



from zope.interface import Attribute
from zope.interface import Interface


class IValidator(Interface):

    name = Attribute("name of the validator")
    title = Attribute("title or name of the validator")
    description = Attribute("description of the validator")

    def __call__(value, *args, **kwargs):
        """return True if valid, error string if not"""


class IValidationChain(IValidator):
    """Marker interface for a chain"""



from Products.validation.i18n import PloneMessageFactory as _
from Products.validation.validators.RangeValidator import RangeValidator
from Products.validation.validators.RegexValidator import RegexValidator


# protocols for isURL validator, the secure (*s) variants are automagically
# added
protocols = (
    "http",
    "ftp",
    "irc",
    "news",
    "imap",
    "gopher",
    "jabber",
    "webdav",
    "smb",
    "fish",
    "ldap",
    "pop3",
    "smtp",
    "sftp",
    "ssh",
    "feed",
)

# email re w/o leading '^'
EMAIL_RE = r"([0-9a-zA-Z_&.'+-]+!)*[0-9a-zA-Z_&.'+-]+@(([0-9a-zA-Z]([0-9a-zA-Z-]*[0-9a-z-A-Z])?\.)+[a-zA-Z]{2,}|([0-9]{1,3}\.){3}[0-9]{1,3})$"

baseValidators = [
    RangeValidator("inNumericRange", title="", description=""),
    RegexValidator(
        "isDecimal",
        r"^([+-]?)(?=\d|(\.|\,)\d)\d*((\,|\.)\d*)?([Ee]([+-]?\d+))?$",
        title="",
        description="",
        errmsg=_("is not a decimal number."),
    ),
    RegexValidator(
        "isInt",
        r"^([+-])?\d+$",
        title="",
        description="",
        errmsg=_("is not an integer."),
    ),
    RegexValidator(
        "isPrintable",
        r"[a-zA-Z0-9\s]+$",
        title="",
        description="",
        errmsg=_("contains unprintable characters"),
    ),
    RegexValidator(
        "isSSN",
        r"^\d{9}$",
        title="",
        description="",
        errmsg=_("is not a well formed SSN."),
    ),
    RegexValidator(
        "isUSPhoneNumber",
        r"^\d{10}$",
        ignore=r"[\(\)\-\s]",
        title="",
        description="",
        errmsg=_("is not a valid us phone number."),
    ),
    RegexValidator(
        "isInternationalPhoneNumber",
        r"^\d+$",
        ignore=r"[\(\)\-\s\+]",
        title="",
        description="",
        errmsg=_("is not a valid international phone number."),
    ),
    RegexValidator(
        "isZipCode",
        r"^(\d{5}|\d{9})$",
        title="",
        description="",
        errmsg=_("is not a valid zip code."),
    ),
    RegexValidator(
        "isURL",
        r"(%s)s?://[^\s\r\n]+" % "|".join(protocols),
        title="",
        description="",
        errmsg=_("is not a valid url."),
    ),
    RegexValidator(
        "isEmail",
        "^" + EMAIL_RE,
        title="",
        description="",
        errmsg=_("is not a valid email address."),
    ),
    RegexValidator(
        "isMailto",
        "^mailto:" + EMAIL_RE,
        title="",
        description="",
        errmsg=_("is not a valid email address."),
    ),
    RegexValidator(
        "isUnixLikeName",
        r"^[A-Za-z][\w\d\-\_]{0,7}$",
        title="",
        description="",
        errmsg=_("this name is not a valid identifier"),
    ),
]

__all__ = ("baseValidators",)



# This module gathers a list of validators, and registers them to the service.
#
# Three of these are backwards compatibility imports, marked with 'noqa F401'
# for flake8.  Of these, the RangeValidator and RegexValidator are included in
# baseValidators, so that is fine.
# ExpressionValidator is not included.  This is because you need to create an
# instance of a validator class and register this.  The ExpressionValidator
# class is fine as base for your custom validators, but no one thought of
# a default instance to add here.
from Products.validation.validators.BaseValidators import baseValidators
from Products.validation.validators.EmptyValidator import (
    validatorList as emptyValidators,
)
from Products.validation.validators.ExpressionValidator import (  # noqa F401
    ExpressionValidator,
)
from Products.validation.validators.IdValidator import validatorList as idValidators
from Products.validation.validators.RangeValidator import RangeValidator  # noqa F401
from Products.validation.validators.RegexValidator import RegexValidator  # noqa F401
from Products.validation.validators.SupplValidators import (
    validatorList as supplValidators,
)


validators = []
validators.extend(baseValidators)
validators.extend(emptyValidators)
validators.extend(supplValidators)
validators.extend(idValidators)


def initialize(service):
    for validator in validators:
        service.register(validator)



from pathlib import Path
from setuptools import setup


version = "4.0.1"

long_description = (
    f"{Path('README.rst').read_text()}\n{Path('CHANGES.rst').read_text()}\n"
)

setup(
    name="Products.validation",
    version=version,
    description="Data validation package for Zope",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    classifiers=[
        "Development Status :: 6 - Mature",
        "Framework :: Zope",
        "Framework :: Zope :: 5",
        "Operating System :: OS Independent",
        "Framework :: Plone",
        "Framework :: Plone :: 6.2",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
    ],
    keywords="Zope validation regex email",
    author="Benjamin Saller",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/Products.validation",
    license="GPL",
    include_package_data=True,
    zip_safe=False,
    extras_require={},
    python_requires=">=3.10",
    install_requires=[
        "setuptools",
        "Zope",
    ],
)



from unittest import TestSuite

import doctest


def test_suite():
    suite = TestSuite()
    suite.addTest(
        doctest.DocTestSuite("Products.validation.validators.ExpressionValidator")
    )
    return suite



from Products.validation.chain import V_REQUIRED
from Products.validation.chain import V_SUFFICIENT
from Products.validation.chain import ValidationChain
from Products.validation.config import validation
from Products.validation.exceptions import AlreadyRegisteredValidatorError
from Products.validation.exceptions import FalseValidatorError
from Products.validation.exceptions import UnknowValidatorError



from Products.validation.config import validation as validationService
from Products.validation.exceptions import ValidatorError
from Products.validation.interfaces.IValidator import IValidationChain
from zope.interface import implementer


V_REQUIRED = 1
V_SUFFICIENT = 2
TupleType = type((0, 1))
ListType = type([])


@implementer(IValidationChain)
class ValidationChain:
    """ """

    def __init__(self, name, title="", description="", validators=(), register=False):
        self.name = name
        self.title = title or name
        self.description = description
        self._v_mode = []
        self._chain = []

        if type(validators) not in (TupleType, ListType):
            validators = (validators,)
        for validator in validators:
            if type(validator) in (TupleType, ListType):
                self.append(validator[0], validator[1])
            else:
                self.appendRequired(validator)

        if register:
            validationService.register(self)

    def __repr__(self):
        """print obj support"""
        map = {V_REQUIRED: "V_REQUIRED", V_SUFFICIENT: "V_SUFFICIENT"}
        val = []
        for validator, mode in self:
            name = validator.name
            val.append(f"('{name}', {map.get(mode)})")
        return "(%s)" % ", ".join(val)

    def __len__(self):
        """len(obj) support"""
        return len(self._chain)

    def __iter__(self):
        """Python 2.3 for i in x support"""
        return iter(zip(self._chain, self._v_mode))

    def __eq__(self, other):
        if isinstance(other, ValidationChain):
            value = repr(other)
        else:
            value = other
        return repr(self) == value

    def __lt__(self, other):
        if isinstance(other, ValidationChain):
            value = repr(other)
        else:
            value = other
        return repr(self) < value

    def __gt__(self, other):
        if isinstance(other, ValidationChain):
            value = repr(other)
        else:
            value = other
        return repr(self) > value

    def __getitem__(self, idx):
        """self[idx] support and Python 2.1 for i in x support"""
        return self._chain[idx], self._v_mode[idx]

    def append(self, id_or_obj, mode=V_REQUIRED):
        """Appends a validator"""
        validator = self.setValidator(id_or_obj)
        self.setMode(validator, mode)

    def appendRequired(self, id_or_obj):
        """Appends a validator as required"""
        self.append(id_or_obj, mode=V_REQUIRED)

    def appendSufficient(self, id_or_obj):
        """Appends a validator as sufficient"""
        self.append(id_or_obj, mode=V_SUFFICIENT)

    def insert(self, id_or_obj, mode=V_REQUIRED, position=0):
        """Inserts a validator at position (default 0)"""
        validator = self.setValidator(id_or_obj, position=position)
        self.setMode(validator, mode, position=position)

    def insertRequired(self, id_or_obj, position=0):
        """Inserts a validator as required at position (default 0)"""
        self.insert(id_or_obj, mode=V_REQUIRED, position=0)

    def insertSufficient(self, id_or_obj, position=0):
        """Inserts a validator as sufficient at position (default 0)"""
        self.insert(id_or_obj, mode=V_SUFFICIENT, position=0)

    def setMode(self, validator, mode, position=None):
        """Set mode"""
        # validator not required
        if position is None:
            self._v_mode.append(mode)
        else:
            self._v_mode.insert(position, mode)

    def setValidator(self, id_or_obj, position=None):
        """Set validator"""
        validator = validationService.validatorFor(id_or_obj)

        if position is None:
            self._chain.append(validator)
        else:
            self._chain.insert(position, validator)

        return validator

    def __call__(self, value, *args, **kwargs):
        """Do validation"""
        results = {}
        failed = False
        if len(self) == 1:
            mode = [m for (v, m) in self][0]
            if mode == V_SUFFICIENT:
                # There is only one validator and its mode is
                # 'sufficient' which means it does not have to
                # validate.  So we cut the validation short.
                return True  # validation was successful

        for validator, mode in self:
            name = validator.name
            result = validator(value, *args, **kwargs)
            if result is True:
                if mode == V_SUFFICIENT:
                    return True  # validation was successful
                elif mode == V_REQUIRED:
                    continue  # go on
                else:
                    raise ValidatorError(f"Unknown mode {mode}")
            else:
                if mode == V_SUFFICIENT:
                    if isinstance(result, (str, bytes)):
                        # don't log if validator doesn't return an error msg
                        results[name] = result
                    continue  # no fatal error, go on
                elif mode == V_REQUIRED:
                    if isinstance(result, (str, bytes)):
                        # don't log if validator doesn't return an error msg
                        results[name] = result
                    failed = True
                    break  # fatal error, stop and fail
                else:
                    raise ValidatorError(f"Unknown mode {mode}")

        if failed:
            return "\n".join(
                [
                    # '%s: %s' % (name, res)
                    "%s" % res
                    for name, res in results.items()
                ]
            )
        else:
            return True



from Products.validation.i18n import PloneMessageFactory as _
from Products.validation.i18n import recursiveTranslate
from Products.validation.i18n import safe_unicode
from Products.validation.interfaces.IValidator import IValidator
from zope.interface import implementer

import re


def ignoreRE(value, expression):
    ignore = re.compile(expression)
    return ignore.sub("", value)


@implementer(IValidator)
class RegexValidator:

    def __init__(self, name, *args, **kw):
        self.name = name
        self.title = kw.get("title", name)
        self.description = kw.get("description", "")
        self.errmsg = kw.get("errmsg", "fails tests of %s" % name)
        self.regex_strings = args
        self.ignore = kw.get("ignore", None)
        self.regex = []
        self.compileRegex()

    def compileRegex(self):
        for r in self.regex_strings:
            self.regex.append(re.compile(r))

    def __getstate__(self):
        """Because copy.deepcopy and pickle.dump cannot pickle a regex pattern
        I'm using the getstate/setstate hooks to set self.regex to []
        """
        d = self.__dict__.copy()
        d["regex"] = []
        return d

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.compileRegex()

    def __call__(self, value, *args, **kwargs):
        if not isinstance(value, str):
            msg = _(
                "Validation failed($name): $value of type $type, expected 'string'",
                mapping={
                    "name": safe_unicode(self.name),
                    "value": safe_unicode(value),
                    "type": safe_unicode(type(value)),
                },
            )
            return recursiveTranslate(msg, **kwargs)

        ignore = kwargs.get("ignore", None)
        if ignore:
            value = ignoreRE(value, ignore)
        elif self.ignore:
            value = ignoreRE(value, self.ignore)

        for r in self.regex:
            m = r.match(value)
            if not m:
                msg = _(
                    "Validation failed($name): '$value' $errmsg",
                    mapping={
                        "name": safe_unicode(self.name),
                        "value": safe_unicode(value),
                        "errmsg": safe_unicode(self.errmsg),
                    },
                )

                return recursiveTranslate(msg, **kwargs)
        return 1
